@@ -2,7 +2,6 @@ import logging
 import os
 import traceback
 import uuid
-from collections.abc import Sequence
 from typing import cast
 
 import nbformat as nbf
@@ -24,12 +23,7 @@ from fastapi.staticfiles import StaticFiles
 from pydantic import BaseModel
 from sqlmodel import Session, select
 from unitxt.inference import InferenceEngine, MockInferenceEngine
-from unitxt.llm_as_judge import (
-    DIRECT_CRITERIA,
-    PAIRWISE_CRITERIA,
-    Criteria,
-    EvaluatorTypeEnum,
-)
+from unitxt.llm_as_judge import DIRECT_CRITERIA, PAIRWISE_CRITERIA, EvaluatorTypeEnum
 
 # Logging req/resp
 from .actions_logger import LoggingRoute
@@ -52,6 +46,7 @@ from .api_types import (
     FixInstanceResponse,
     InstanceResultDTO,
     NotebookParams,
+    PairwiseInstanceDTO,
     PersonaEnum,
     PutTestCaseBody,
     SyntheticExampleGenerationRequest,
@@ -69,10 +64,13 @@ from .const import (
 )
 from .database import engine  # Assumes you have engine/session setup
 from .extended_unitxt import EXTENDED_EVALUATORS_METADATA
-from .judges import SimpleDirectJudge
-from .judges.const import DEFAULT_JUDGE_INFERENCE_PARAMS
-from .judges.types import DirectInstance, PairwiseInstance
-from .judges.unitxt_judges import GraniteGuardianJudge, UnitxtPairwiseJudge
+from .judges import (
+    DEFAULT_JUDGE_INFERENCE_PARAMS,
+    Criteria,
+    DirectJudge,
+    GraniteGuardianJudge,
+    PairwiseJudge,
+)
 from .model import AppUser, StoredTestCase
 from .notebook_generation import DirectEvaluationNotebook, PairwiseEvaluationNotebook
 
@@ -236,7 +234,7 @@ def get_criterias() -> dict[str, list[CriteriaWithOptionsDTO] | list[CriteriaDTO
                     CriteriaOptionDTO(name=o.name, description=o.description)
                     for o in c.options
                 ],
-                prediction_field=cast(str, c.prediction_field),
+                to_evaluate_field=cast(str, c.prediction_field),
                 context_fields=cast(list[str], c.context_fields),
             )
             for c in DIRECT_CRITERIA
@@ -245,7 +243,7 @@ def get_criterias() -> dict[str, list[CriteriaWithOptionsDTO] | list[CriteriaDTO
             CriteriaDTO(
                 name=c.name,
                 description=c.description,
-                prediction_field=cast(str, c.prediction_field),
+                to_evaluate_field=cast(str, c.prediction_field),
                 context_fields=cast(list[str], c.context_fields),
             )
             for c in PAIRWISE_CRITERIA
@@ -309,10 +307,19 @@ async def evaluate(
                 **DEFAULT_JUDGE_INFERENCE_PARAMS,
             },
         )
-        if req.criteria.prediction_field is None or req.criteria.context_fields is None:
+        if (
+            req.criteria.to_evaluate_field is None
+            or req.criteria.context_fields is None
+        ):
             raise ValueError(
                 "EvalAssist uses the new LLM Judge API, where the predictions and context fields are provided in the criteria definition. Make sure to adhere to it."
             )
+
+        instances = [
+            dto_instance.to_instance(req.criteria.to_evaluate_field)
+            for dto_instance in req.instances
+        ]
+
         if req.type == EvaluatorTypeEnum.DIRECT:
             criteria = criteria_with_options_DTO_to_BO_mapper(
                 cast(CriteriaWithOptionsDTO, req.criteria)
@@ -320,28 +327,30 @@ async def evaluate(
             if evaluator_name.name.startswith("GRANITE_GUARDIAN"):
                 evaluator = GraniteGuardianJudge(inference_engine=inference_engine)
             else:
-                evaluator = SimpleDirectJudge(
+                evaluator = DirectJudge(
                     inference_engine=inference_engine,
                     generate_feedback=True,
+                    check_positional_bias=True,
                 )
             per_instance_result = evaluator.evaluate(
-                instances=cast(Sequence[DirectInstance], req.instances),
+                instances=instances,
                 criteria=criteria,
-                check_positional_bias=True,
             )
         else:
             criteria = Criteria(
                 name=req.criteria.name,
                 description=req.criteria.description,
-                prediction_field=req.criteria.prediction_field,
+                to_evaluate_field=req.criteria.to_evaluate_field,
                 context_fields=req.criteria.context_fields,
             )
-            evaluator = UnitxtPairwiseJudge(inference_engine)
+            evaluator = PairwiseJudge(
+                inference_engine=inference_engine,
+                check_positional_bias=True,
+            )
 
             per_instance_result = evaluator.evaluate(
-                instances=cast(Sequence[PairwiseInstance], req.instances),
+                instances=instances,
                 criteria=criteria,
-                check_positional_bias=True,
             )
 
         # Add the id to each instance result
@@ -475,22 +484,7 @@ def create_user_if_not_exist(
 @router.get("/benchmarks/")
 def get_benchmarks():
     benchmarks_results = get_all_benchmarks()
-    fetched_results = []
-    for benchmark_name, benchmark_data in benchmarks_results.items():
-        fetched_criteria_benchmarks = []
-        for criteria_name, criteria_benchmark_data in benchmark_data[
-            "criteria_benchmarks"
-        ].items():
-            criteria_benchmark_data["evaluator_benchmarks"] = list(
-                criteria_benchmark_data["evaluator_benchmarks"].values()
-            )
-            fetched_criteria_benchmarks.append(criteria_benchmark_data)
-        benchmark_data["criteria_benchmarks"] = list(
-            benchmark_data["criteria_benchmarks"].values()
-        )
-        benchmark_data["name"] = benchmark_data["display_name"]
-        fetched_results.append(benchmark_data)
-    return fetched_results
+    return benchmarks_results
 
 
 def cleanup_file(filepath: str):
@@ -575,15 +569,30 @@ def download_test_case(params: DownloadTestCaseBody, background_tasks: Backgroun
 
 @router.post("/download-test-data/")
 def download_test_data(params: DownloadTestDataBody, background_tasks: BackgroundTasks):
-    # in the csv I want the instances:
+    prediction_rows = []
+    if isinstance(params.instances[0], PairwiseInstanceDTO):
+        prediction_rows = [
+            {
+                f"{params.to_evaluate_field} {i + 1}": response
+                for i, response in enumerate(
+                    cast(PairwiseInstanceDTO, instance).responses
+                )
+            }
+            for instance in params.instances
+        ]
+    else:
+        prediction_rows = [
+            {params.to_evaluate_field: cast(DirectInstanceDTO, instance).response}
+            for instance in params.instances
+        ]
+
     instances = params.instances
     rows = [
         {
-            **i.context,
-            params.prediction_field: i.get_prediction(),
-            "expected_result": i.expected_result,
+            **to_evaluate_fields,
+            **instance.context,
         }
-        for i in instances
+        for instance, to_evaluate_fields in zip(instances, prediction_rows)
     ]
 
     df = pd.DataFrame(rows)
@@ -657,7 +666,8 @@ def get_synthetic_examples(params: SyntheticExampleGenerationRequest):
             per_criteria_option_count=params.per_criteria_option_count,
             borderline_count=params.borderline_count,
         )
-        return generator.generate()
+        result = generator.generate()
+        return result
 
     return run()
 
@@ -700,7 +710,11 @@ async def fix_instance(
         llm_provider_credentials=req.llm_provider_credentials,
         custom_params=SYNTHETIC_DATA_GENERATION_PARAMS,
     )
-    context = "\n".join([f"{k}: {v}" for k, v in req.instance.context.items()])
+    context = (
+        "\n".join([f"{k}: {v}" for k, v in req.instance.context.items()])
+        if req.instance.context is not None
+        else []
+    )
     if len(context):
         context = f"### Context\n{context}\n"
     prompt = f"""
@@ -714,13 +728,13 @@ Fix the following text based on the feedback
 ### Feedback
 {req.result.feedback}
 
-Only respond with the fixed text, not titles.
+Only respond with the fixed text, not titles. Keep the text length if possible.
 
 Fixed text:
 
 """
     result = inference_engine([{"source": prompt}])
-    return FixInstanceResponse(fixed_response=result[0])
+    return FixInstanceResponse(fixed_response=cast(str, result[0]))
 
 
 app.include_router(router, prefix="/api")
