@@ -2,21 +2,22 @@ import logging
 import os
 import uuid
 from textwrap import dedent
+from typing import cast
 
-from langchain.output_parsers import (
-    OutputFixingParser,
-    ResponseSchema,
-    StructuredOutputParser,
+from evalassist.judges.batch_repair_parser import BatchRepairParser
+from evalassist.judges.utils import (
+    build_format_instructions,
+    generate_dynamic_pydantic_model,
 )
-from langchain.prompts import PromptTemplate
-from langchain_core.runnables import RunnableLambda
+from evalassist.main import DirectInstanceDTO
+from langchain_core.prompts import PromptTemplate
+from pydantic import Field
 from unitxt.inference import InferenceEngine
 
 from ..api_types import (
     CriteriaWithOptionsDTO,
     DirectActionTypeEnum,
     DirectAIActionRequest,
-    DirectInstanceDTO,
     DomainEnum,
     GenerationLengthEnum,
     PersonaEnum,
@@ -68,194 +69,190 @@ class DirectActionGenerator:
             DirectActionTypeEnum.LONGER: "elaborated",
         }
 
+        self.parser = BatchRepairParser(
+            inference_engine=self.inference_engine,
+            max_retries=3,
+            on_generation_failure="random",
+        )
+
     def generate(self, direct_ai_action: DirectAIActionRequest):
         if self.action == DirectActionTypeEnum.CUSTOM:
-            response_schemas = [
-                ResponseSchema(
-                    name="response",
-                    description="the selection to apply the action to",
-                )
-            ]
-
-            output_parser = StructuredOutputParser.from_response_schemas(
-                response_schemas
-            )
             action_tag = "<custom_action>"
-            format_instructions = output_parser.get_format_instructions()
+            dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        "response",
+                        str,
+                        Field(
+                            ...,
+                            description="the selection to apply the action to",
+                        ),
+                        [],
+                    ),
+                ],
+            )
+            format_instructions = build_format_instructions(dynamic_model)
             text_with_selection = direct_ai_action.text.replace(
                 direct_ai_action.selection,
                 action_tag + direct_ai_action.selection + action_tag,
             )
             # prompt templates
-            system_prompt_template = PromptTemplate(
-                input_variables=[
-                    "text_with_selection",
-                    "selection",
-                ],
-                partial_variables={
-                    "format_instructions": format_instructions,
-                    "action_description": self.prompt,
-                    "action_tag": action_tag,
-                },
-                template=dedent(
-                    """\
-                    You will be provided with:
+            system_prompt_template = dedent(
+                """\
+                You will be provided with:
 
-                    - A selected text
+                - A selected text
 
-                    - A text containing that selection, with the selection marked using {action_tag} tags
+                - A text containing that selection, with the selection marked using {action_tag} tags
 
-                    Your task is to {action_description} the selected text such that:
+                Your task is to {action_description} the selected text such that:
 
-                    - It preserves the original meaning and intent
+                - It preserves the original meaning and intent
 
-                    - It fits seamlessly into the original text, both semantically and grammatically
+                - It fits seamlessly into the original text, both semantically and grammatically
 
-                    ✅ The generated selection must not disrupt the sentence structure or introduce grammatical errors (e.g., missing prepositions or incorrect tense).
-                    🚫 Do not introduce any new information that is not present in the original text.
+                ✅ The generated selection must not disrupt the sentence structure or introduce grammatical errors (e.g., missing prepositions or incorrect tense).
+                🚫 Do not introduce any new information that is not present in the original text.
 
-                    Selection:
-                    {selection}
+                Selection:
+                {selection}
 
-                    Text with selection (wrapped in-between {action_tag} tags):
-                    {text_with_selection}
+                Text with selection (wrapped in-between {action_tag} tags):
+                {text_with_selection}
 
-                    {format_instructions}
-                    Don't forget to enclose the response value in double quotes.
-                """,
-                ),
+                {format_instructions}
+                Don't forget to enclose the response value in double quotes.
+            """,
             )
 
             system_prompt = system_prompt_template.format(
                 text_with_selection=text_with_selection,
                 selection=direct_ai_action.selection,
+                format_instructions=format_instructions,
+                action_description=self.prompt,
+                action_tag=action_tag,
             )
         elif self.action == DirectActionTypeEnum.REGENERATE:
-            response_schemas = [
-                ResponseSchema(
-                    name="response",
-                    description="the selection to regenerate",
-                )
-            ]
-
-            output_parser = StructuredOutputParser.from_response_schemas(
-                response_schemas
+            dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        "response",
+                        str,
+                        Field(
+                            ...,
+                            description="the selection to regenerate",
+                        ),
+                        [],
+                    ),
+                ],
             )
+            format_instructions = build_format_instructions(dynamic_model)
             action_str = direct_ai_action.action.value.lower()
             action_tag = "<regenerate>"
-            format_instructions = output_parser.get_format_instructions()
             text_with_selection = direct_ai_action.text.replace(
                 direct_ai_action.selection,
                 action_tag + direct_ai_action.selection + action_tag,
             )
             # prompt templates
-            system_prompt_template = PromptTemplate(
-                input_variables=[
-                    "text_with_selection",
-                    "selection",
-                ],
-                partial_variables={
-                    "format_instructions": format_instructions,
-                },
-                template=dedent("""\
-                    You will be provided with:
-                    - A selected text
-                    - A text containing that selection, with the selection marked using <regenerate> tags
-                    - Your task is to substitute the selected text with a counterfactual example to diversify perspective, demographic, or approach. It should fit seamlessly into the original text. The regenerated selection must not disrupt the sentence structure or introduce grammatical errors (e.g., missing prepositions or incorrect tense).
-                    - Examples: “toddler” changed to “adult”, “terrorist” changed to “diplomat”, “men” changed to “women”, “easy” changed to “difficult”, “great” changed to “poor”
-      
-                    Selection:
-                    {selection}
+            system_prompt_template = dedent("""\
+                You will be provided with:
+                - A selected text
+                - A text containing that selection, with the selection marked using <regenerate> tags
+                - Your task is to substitute the selected text with a counterfactual example to diversify perspective, demographic, or approach. It should fit seamlessly into the original text. The regenerated selection must not disrupt the sentence structure or introduce grammatical errors (e.g., missing prepositions or incorrect tense).
+                - Examples: “toddler” changed to “adult”, “terrorist” changed to “diplomat”, “men” changed to “women”, “easy” changed to “difficult”, “great” changed to “poor”
 
-                    Text with selection (wrapped in-between <regenerate> tags):
-                    {text_with_selection}
+                Selection:
+                {selection}
 
-                    {format_instructions}
-                    Don't forget to enclose the response value in double quotes.
-                    """),
-            )
+                Text with selection (wrapped in-between <regenerate> tags):
+                {text_with_selection}
+
+                {format_instructions}
+                Don't forget to enclose the response value in double quotes.
+                """)
             system_prompt = system_prompt_template.format(
                 text_with_selection=text_with_selection,
                 selection=direct_ai_action.selection,
+                format_instructions=format_instructions,
             )
         else:
-            response_schemas = [
-                ResponseSchema(
-                    name="response",
-                    description=f"the selection to {self.action.value.lower()}",
-                )
-            ]
-
-            output_parser = StructuredOutputParser.from_response_schemas(
-                response_schemas
+            dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        "response",
+                        str,
+                        Field(
+                            ...,
+                            description=f"the selection to {self.action.value.lower()}",
+                        ),
+                        [],
+                    ),
+                ],
             )
+            format_instructions = build_format_instructions(dynamic_model)
             action_str = direct_ai_action.action.value.lower()
             action_tag = f"<{action_str}>"
-            format_instructions = output_parser.get_format_instructions()
             text_with_selection = direct_ai_action.text.replace(
                 direct_ai_action.selection,
                 action_tag + direct_ai_action.selection + action_tag,
             )
             # prompt templates
-            system_prompt_template = PromptTemplate(
-                input_variables=[
-                    "text_with_selection",
-                    "selection",
-                ],
-                partial_variables={
-                    "action_third_person": self.action_third_person_dict[self.action],
-                    "action_infinitive": self.action_infinitive_person_dict[
-                        self.action
-                    ],
-                    "action_past": self.action_past_dict[self.action],
-                    "action_tag": action_tag,
-                    "format_instructions": format_instructions,
-                },
-                template=dedent("""\
-                    You will be provided with:
+            system_prompt_template = dedent("""\
+                You will be provided with:
 
-                    - A selected text
+                - A selected text
 
-                    - A text containing that selection, with the selection marked using {action_tag} tags
+                - A text containing that selection, with the selection marked using {action_tag} tags
 
-                    Your task is {action_infinitive} the selected text such that:
+                Your task is {action_infinitive} the selected text such that:
 
-                    - It preserves the original meaning and intent
+                - It preserves the original meaning and intent
 
-                    - It fits seamlessly into the original text, both semantically and grammatically
+                - It fits seamlessly into the original text, both semantically and grammatically
 
-                    ✅ The {action_past} selection must not disrupt the sentence structure or introduce grammatical errors (e.g., missing prepositions or incorrect tense).
-                    🚫 Do not introduce any new information that is not present in the original text.
+                ✅ The {action_past} selection must not disrupt the sentence structure or introduce grammatical errors (e.g., missing prepositions or incorrect tense).
+                🚫 Do not introduce any new information that is not present in the original text.
 
-                    - if the selection is equal to the whole text, your task is {action_infinitive} the whole text.
-                    - Examples: “toddler” changed to “kid”, “terrorist” changed to “extremist”, “men” changed to “human”, “easy” changed to “simple”, “great” changed to “excellent”
+                - If the selection is equal to the whole text, your task is {action_infinitive} the whole text.
+                - Examples: “toddler” changed to “kid”, “terrorist” changed to “extremist”, “men” changed to “human”, “easy” changed to “simple”, “great” changed to “excellent”
 
-                    Selection:
-                    {selection}
+                Selection:
+                {selection}
 
-                    Text with selection (wrapped in-between {action_tag} tags):
-                    {text_with_selection}
+                Text with selection (wrapped in-between {action_tag} tags):
+                {text_with_selection}
 
-                    {format_instructions}
-                    Don't forget to enclose the response value in double quotes.
-                    """),
-            )
+                {format_instructions}
+                Don't forget to enclose the response value in double quotes.
+                """)
 
             system_prompt = system_prompt_template.format(
                 text_with_selection=text_with_selection,
                 selection=direct_ai_action.selection,
+                action_third_person=self.action_third_person_dict[self.action],
+                action_infinitive=self.action_infinitive_person_dict[self.action],
+                action_past=self.action_past_dict[self.action],
+                action_tag=action_tag,
+                format_instructions=format_instructions,
             )
 
         prompt = system_prompt
 
         logger.debug(f"Direct AI action prompt:\n{prompt}")
 
-        response = self.inference_engine.infer([{"source": prompt}])[0]
-        logger.debug(f"Direct AI action unparsed response:\n{response}")
+        unparsed_response: str = cast(
+            str, self.inference_engine.infer([{"source": prompt}])[0]
+        )
+        logger.debug(f"Direct AI action unparsed response:\n{unparsed_response}")
 
-        parsed_response = output_parser.parse(response)["response"]
-
-        return parsed_response
+        parsed_responses, metadata = self.parser.parse_and_repair(
+            [unparsed_response], [dynamic_model]
+        )
+        parsed_response = parsed_responses[0]
+        return parsed_response.model_dump()["response"]
 
 
 class Generator:
@@ -287,13 +284,6 @@ class Generator:
             > 0
         )
 
-        def llm_invoke(text: str) -> str:
-            # call your custom model here and return the raw text
-            response = self.inference_engine.infer([{"source": text.to_string()}])[0]
-            return response
-
-        self.llm_runnable = RunnableLambda(llm_invoke)
-
         system_prompt_input_variables = [
             "dimension",
             "dimension_description",
@@ -305,21 +295,28 @@ class Generator:
             "response_name",
         ]
 
-        if self.task == TaskEnum.QUESTION_ANSWERING:
-            # response schema
-            response_schema = [
-                ResponseSchema(
-                    name=self.criteria.prediction_field,
-                    description="the answer to the question",
-                ),
-            ]
-            self.output_parser = OutputFixingParser.from_llm(
-                parser=StructuredOutputParser.from_response_schemas(response_schema),
-                llm=self.llm_runnable,
-                max_retries=3,
-            )
+        self.parser = BatchRepairParser(
+            inference_engine=self.inference_engine,
+            max_retries=3,
+            on_generation_failure="raise",
+        )
 
-            self.format_instructions = self.output_parser.get_format_instructions()
+        if self.task == TaskEnum.QUESTION_ANSWERING:
+            self.dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        self.criteria.to_evaluate_field,
+                        str,
+                        Field(
+                            ...,
+                            description="the answer to the question",
+                        ),
+                        [],
+                    ),
+                ],
+            )
+            self.format_instructions = build_format_instructions(self.dynamic_model)
 
             # prompt templates
             self.system_prompt_template = PromptTemplate(
@@ -337,7 +334,7 @@ class Generator:
                     Important:
                     {domain_section}{persona_section}{generation_length_section}- Focus exclusively on the specified dimension and target
                     - Make sure your answer clearly demonstrates the described characteristics
-                    - Do not mention the criteria in your answer - simply generate an answer to the question that embodies the characteristics
+                    - Do not mention the criteria in your answer - Simply generate an answer to the question that embodies the characteristics
                     """),
             )
 
@@ -347,21 +344,21 @@ class Generator:
                 partial_variables={"format_instructions": self.format_instructions},
             )
         elif self.task == TaskEnum.SUMMARIZATION:
-            # response schema
-            response_schema = [
-                ResponseSchema(
-                    name=self.criteria.context_fields[0],
-                    description=f"the {self.criteria.context_fields[0]}'s summary",
-                ),
-            ]
-
-            self.output_parser = OutputFixingParser.from_llm(
-                parser=StructuredOutputParser.from_response_schemas(response_schema),
-                llm=self.llm_runnable,
-                max_retries=3,
+            self.dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        self.criteria.context_fields[0],
+                        str,
+                        Field(
+                            ...,
+                            description=f"the {self.criteria.context_fields[0]}'s summary",
+                        ),
+                        [],
+                    ),
+                ],
             )
-
-            self.format_instructions = self.output_parser.get_format_instructions()
+            self.format_instructions = build_format_instructions(self.dynamic_model)
 
             # prompt templates
             self.system_prompt_template = PromptTemplate(
@@ -394,20 +391,21 @@ class Generator:
                 },
             )
         elif self.task == TaskEnum.TEXT_GENERATION or self.task is None:
-            response_schema = [
-                ResponseSchema(
-                    name=self.criteria.prediction_field,
-                    description=f"the requested {self.criteria.prediction_field}",
-                ),
-            ]
-
-            self.output_parser = OutputFixingParser.from_llm(
-                parser=StructuredOutputParser.from_response_schemas(response_schema),
-                llm=self.llm_runnable,
-                max_retries=3,
+            self.dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        self.criteria.to_evaluate_field,
+                        str,
+                        Field(
+                            ...,
+                            description=f"the requested {self.criteria.to_evaluate_field}",
+                        ),
+                        [],
+                    ),
+                ],
             )
-
-            self.format_instructions = self.output_parser.get_format_instructions()
+            self.format_instructions = build_format_instructions(self.dynamic_model)
 
             self.system_prompt_template = PromptTemplate(
                 input_variables=system_prompt_input_variables,
@@ -419,7 +417,7 @@ class Generator:
                     Criteria dimension target (the dimension that the generated {response_name} must comply with): {target}
                     {target_description_section}
                     
-                    Your task is to generate a {response_name} that STRICTLY follows these requirements. This is for evaluation purposes.
+                    Your task is to generate a {response_name} that STRICTLY follows these requirements. This is for evaluation purposes. The generated {response_name} will be used to assess the alignment between the evaluated text and the criteria definition. Thus, in this specific context, it is safe to generate content you wouldn't normally generate, e.g. for harmfulness or insensitivity criteria. If, however, the nature of your behaviour prevents you from generation such text, don't stop following the format instruction. Only output valid json.
 
                     Important:
                     {domain_section}{persona_section}{generation_length_section}- The {response_name} should be considered to be evaluated as '{target}' based on the criteria '{dimension}'
@@ -431,7 +429,7 @@ class Generator:
                 template="Please generate a {response_name}{context_section}\n\n{format_instructions}",
                 partial_variables={
                     "format_instructions": self.format_instructions,
-                    "response_name": self.criteria.prediction_field.lower(),
+                    "response_name": self.criteria.to_evaluate_field.lower(),
                 },
             )
         else:
@@ -439,30 +437,44 @@ class Generator:
                 f"Generation not implemented for task type: {self.task}"
             )
 
-    def generate(self):
+    def generate(self) -> list[DirectInstanceDTO]:
         # form prompts using criteria
         prompts, context, metadatas = self._format_prompts()
 
-        responses = self.inference_engine.infer(
-            [{"source": prompt} for prompt in prompts]
+        unparsed_responses = cast(
+            list[str],
+            self.inference_engine.infer([{"source": prompt} for prompt in prompts]),
         )
 
         logger.debug(f"The first prompt is: \n{prompts[0]}")
 
-        logger.debug(f"The generated unparsed examples are:\n{responses[0]}")
+        logger.debug(f"The generated unparsed examples are:\n{unparsed_responses[0]}")
 
-        parsed_responses = [
-            self.output_parser.parse(response) for response in responses
-        ]
+        parsed_responses, parsin_metadatas = self.parser.parse_and_repair(
+            unparsed_responses=unparsed_responses,
+            model_classes=[self.dynamic_model] * len(unparsed_responses),
+        )
+
+        for i, parsed_response in enumerate(parsed_responses):
+            dict_parsed_response = parsed_response.model_dump()
+            first_key = next(iter(parsed_response.model_dump().keys()))
+            if not dict_parsed_response[first_key]:
+                setattr(
+                    parsed_response,
+                    first_key,
+                    f"The model couldn't correctly generate an example for this instance. Target criteria option: {metadatas[i]['synthetic_generation']['target_option_name']}",
+                )
 
         logger.debug(f"The generated parsed examples are:\n{parsed_responses[0]}")
 
         instances = [
             DirectInstanceDTO(
                 context=context,
-                # response=parsed_responses[i][self.response_name],
-                response=next(iter(parsed_responses[i].values())),
-                metadata=metadatas[i],
+                response=next(iter(parsed_responses[i].model_dump().values())),
+                metadata={
+                    **metadatas[i],
+                    **parsin_metadatas[i],
+                },
                 id=str(uuid.uuid4()),
             )
             for i in range(len(parsed_responses))
@@ -487,9 +499,8 @@ class Generator:
             self.per_criteria_option_count[criteria_borderline["name"]] = (
                 self.borderline_count
             )
-
         if self.domain is not None:
-            domain_section = f"- The generated {self.criteria.prediction_field.lower()} is going to be evaluated on the {self.domain.value} domain\n"
+            domain_section = f"- The generated {self.criteria.to_evaluate_field.lower()} is going to be evaluated on the {self.domain.value} domain\n"
         else:
             domain_section = ""
 
@@ -499,11 +510,11 @@ class Generator:
             persona_section = ""
 
         if self.generation_length is not None:
-            generation_length_section = f"- The generated {self.criteria.prediction_field.lower()}'s length should be {self.generation_length.value.lower()} ({generation_length_to_sentence_count[self.generation_length]} long).\n"
+            generation_length_section = f"- The generated {self.criteria.to_evaluate_field.lower()}'s length should be {self.generation_length.value.lower()} ({generation_length_to_sentence_count[self.generation_length]} long).\n"
         else:
             generation_length_section = ""
 
-        context = {}
+        context: dict[str, str] = {}
         if self.has_context_variables:
             context = self._generate_synthetic_context()
 
@@ -538,7 +549,7 @@ class Generator:
                 "dimension_description": self.criteria.description,
                 "target": criteria_option_name,
                 "target_description_section": target_description_section,
-                "response_name": self.criteria.prediction_field.lower(),
+                "response_name": self.criteria.to_evaluate_field.lower(),
                 "domain_section": domain_section,
                 "persona_section": persona_section,
                 "generation_length_section": generation_length_section,
@@ -593,7 +604,7 @@ class Generator:
 
         return prompts, context, metadatas
 
-    def _generate_synthetic_context(self):
+    def _generate_synthetic_context(self) -> dict[str, str]:
         if self.task == TaskEnum.SUMMARIZATION:
             system_prompt_template = PromptTemplate(
                 input_variables=[
@@ -617,25 +628,43 @@ class Generator:
             system_prompt = system_prompt_template.format(
                 domain_section=domain_section,
             )
-            response_schemas = [
-                ResponseSchema(name="text", description="the text to generate")
-            ]
+            dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        "text",
+                        str,
+                        Field(
+                            ...,
+                            description="the text to generate",
+                        ),
+                        [],
+                    ),
+                ],
+            )
+            format_instructions = build_format_instructions(self.dynamic_model)
         else:
             system_prompt_template = PromptTemplate(
                 input_variables=[
-                    "criteria",
-                    "criteria_description",
                     "response_name",
                     "context_names",
                     "task_section",
                     "domain_section",
                     "persona_section",
+                    "criteria_name",
+                    "criteria_description",
                 ],
                 template=dedent("""\
-                You will be provided with a list of context variable names. Your task is to generate example values for each of these context variables, considering the following information:
+                You will be provided with a list of context variable names. Your task is to generate example values that simulate *real-life cases* for each of these context variables, considering the following information:
 
                 - Context variables to generate: {context_names}.
-                - The generated context is intended to be used to generate a {response_name}.{task_section}{domain_section}{persona_section}"""),
+                - The generated context is intended to be used to generate a '{response_name}'.{task_section}{domain_section}{persona_section}
+                - For each context variable, generate realistic, natural examples that could be used in a dataset for this evaluation.
+                - Do not describe the variable itself — instead, produce the actual kind of text or data it would contain in real life.
+                - The generated context should make sense as input for generating a {response_name} that will later be evaluated using the {criteria_name} criterion (“{criteria_description}”).
+                - If a variable name is vague (e.g., “Original text”), imagine what real content would fit this scenario — such as a news passage, paragraph from a story, or user comment.
+                - Be concise but natural; summaries may span several sentences, while questions may be short.
+                - Creativity is encouraged, but keep the examples plausible and contextually meaningful."""),
             )
             task_section = (
                 f"\n- The generated context is part of a dataset that conforms to a {self.task.value} task.\n"
@@ -653,48 +682,60 @@ class Generator:
                 else ""
             )
             system_prompt = system_prompt_template.format(
-                context_names=", ".join(self.criteria.context_fields),
-                criteria=self.criteria.name,
+                context_names=", ".join(f"'{s}'" for s in self.criteria.context_fields),
+                criteria_name=self.criteria.name,
                 criteria_description=self.criteria.description,
-                response_name=self.criteria.prediction_field,
+                response_name=self.criteria.to_evaluate_field,
                 task_section=task_section,
                 domain_section=domain_section,
                 persona_section=persona_section,
             )
-            response_schemas = [
-                ResponseSchema(
-                    name=context_name, description=f"the {context_name} to generate"
-                )
-                for context_name in self.criteria.context_fields
-            ]
+            dynamic_model = generate_dynamic_pydantic_model(
+                model_name="structured_output_model",
+                field_definitions=[
+                    (
+                        context_name,
+                        str,
+                        Field(
+                            ...,
+                            description=f"The {context_name} to generate",
+                        ),
+                        [],
+                    )
+                    for context_name in self.criteria.context_fields
+                ],
+            )
 
-        output_parser = OutputFixingParser.from_llm(
-            parser=StructuredOutputParser.from_response_schemas(response_schemas),
-            llm=self.llm_runnable,
-            max_retries=3,
-        )
-
-        format_instructions = output_parser.get_format_instructions()
+        format_instructions = build_format_instructions(dynamic_model)
 
         query_template = PromptTemplate(
             input_variables=[],
-            template="\n{format_instructions}",
+            template=dedent(
+                """\
+                    
+                You must output only valid JSON with no extra text.
+                Use the following schema and formatting rules:
+                {format_instructions}
+                """
+            ),
             partial_variables={"format_instructions": format_instructions},
         )
 
         query = query_template.format()
 
         prompt = system_prompt + query
-        response = self.inference_engine.infer([{"source": prompt}])[0]
+        unparsed_response = cast(
+            str, self.inference_engine.infer([{"source": prompt}])[0]
+        )
 
         logger.debug(f"The prompt used for synthetic generation is:\n{prompt}")
-        logger.debug(f"The synthetic generation response is:\n{response}")
+        logger.debug(f"The synthetic generation response is:\n{unparsed_response}")
 
-        parsed_response = output_parser.parse(response)
-        if self.task == TaskEnum.SUMMARIZATION:
-            parsed_response = {self.criteria.context_fields[0]: parsed_response["text"]}
+        parsed_responses, metadatas = self.parser.parse_and_repair(
+            unparsed_responses=[unparsed_response], model_classes=[dynamic_model]
+        )
 
-        return parsed_response
+        return parsed_responses[0].model_dump()
 
     def _get_borderline_criteria(self, criteria: CriteriaWithOptionsDTO):
         criteria_options = criteria.options
@@ -703,21 +744,31 @@ class Generator:
                 "Need to specify at least two criteria to generate borderline case."
             )
 
-        # response schema
-        response_schemas = [
-            ResponseSchema(name="name", description="the name of borderline criteria"),
-            ResponseSchema(
-                name="description", description="the description of borderline criteria"
-            ),
-        ]
-        criteria_output_parser = OutputFixingParser.from_llm(
-            parser=StructuredOutputParser.from_response_schemas(response_schemas),
-            llm=self.llm_runnable,
-            max_retries=3,
+        dynamic_model = generate_dynamic_pydantic_model(
+            model_name="structured_output_model",
+            field_definitions=[
+                (
+                    "name",
+                    str,
+                    Field(
+                        ...,
+                        description="the name of borderline criteria",
+                    ),
+                    [],
+                ),
+                (
+                    "description",
+                    str,
+                    Field(
+                        ...,
+                        description="the description of borderline criteria",
+                    ),
+                    [],
+                ),
+            ],
         )
-        criteria_format_instructions = criteria_output_parser.get_format_instructions()
+        criteria_format_instructions = build_format_instructions(dynamic_model)
 
-        # form query
         criteria_options_list = [
             f"{option.name}: {option.description}" for option in criteria_options
         ]
@@ -727,13 +778,15 @@ class Generator:
 
         logger.debug(f"The borderline criteria generation prompt is \n{query}")
 
-        res = self.inference_engine.infer([{"source": query}])
-        borderline_criteria_unparsed = res[0]
+        borderline_criteria_unparsed = cast(
+            str, self.inference_engine.infer([{"source": query}])[0]
+        )
         logger.debug(
             f"The unparsed borderline criteria is:\n{borderline_criteria_unparsed}"
         )
-        criteria_other_parsed = criteria_output_parser.parse(
-            borderline_criteria_unparsed
+        borderline_criteria_parsed, metadata = self.parser.parse_and_repair(
+            unparsed_responses=[borderline_criteria_unparsed],
+            model_classes=[dynamic_model],
         )
 
-        return criteria_other_parsed
+        return borderline_criteria_parsed[0].model_dump()
