@@ -6,12 +6,12 @@ import os
 import re
 import time
 import traceback
-from collections.abc import Sequence
 from enum import Enum
 from typing import Any, cast
 
 from botocore.exceptions import NoCredentialsError
 from datasets import IterableDataset
+from evalassist.judges import Criteria, CriteriaOption
 from fastapi import HTTPException
 from ibm_watsonx_ai.wml_client_error import (
     ApiRequestFailure,
@@ -28,13 +28,7 @@ from unitxt.inference import (
     WMLInferenceEngineChat,
     WMLInferenceEngineGeneration,
 )
-from unitxt.llm_as_judge import (
-    Criteria,
-    CriteriaOption,
-    CriteriaWithOptions,
-    EvaluatorNameEnum,
-    ModelProviderEnum,
-)
+from unitxt.llm_as_judge import EvaluatorNameEnum, ModelProviderEnum
 
 from .api_types import CriteriaWithOptionsDTO
 from .const import CUSTOM_MODELS_PATH, EVAL_ASSIST_DIR
@@ -45,7 +39,7 @@ from .extended_unitxt import (
     ExtendedEvaluatorNameEnum,
     ExtendedModelProviderEnum,
 )
-from .judges.types import DirectInstance
+from .judges import Instance
 
 logger = logging.getLogger(__name__)
 
@@ -121,7 +115,6 @@ def get_cross_inference_engine_params(
     provider_specific_args = {}
     inference_engine_params = {
         "max_tokens": 1024,
-        "seed": 42,
         "credentials": credentials.copy(),
         "provider": provider,
     }
@@ -558,20 +551,22 @@ def get_system_version():
 
 def unitxt_dataset_to_evalassist_instances(
     dataset: IterableDataset,
-    criteria: Sequence[Criteria],
-) -> list[DirectInstance]:
+    criteria: list[Criteria],
+) -> list[Instance]:
     if any(
-        criterion.prediction_field is None or criterion.context_fields is None
+        criterion.to_evaluate_field is None or criterion.context_fields is None
         for criterion in criteria
     ):
         raise ValueError(
-            "The criteria.prediction_field is None. It must be set to retrieve the response to evaluate from the task_data"
+            "The criteria.to_evaluate_field is None. It must be set to retrieve the response to evaluate from the task_data"
         )
     task_data_list = [json.loads(d["task_data"]) for d in dataset]
     return [
-        DirectInstance(
-            context={k: task_data[k] for k in cast(list, criterion.context_fields)},
-            response=task_data[criterion.prediction_field],
+        Instance(
+            fields={
+                **{k: task_data[k] for k in cast(list, criterion.context_fields)},
+                criterion.to_evaluate_field: task_data[criterion.to_evaluate_field],
+            },
         )
         for task_data, criterion in zip(task_data_list, criteria)
     ]
@@ -605,15 +600,15 @@ def get_inference_engine_from_judge_metadata(
 
 def criteria_with_options_DTO_to_BO_mapper(
     criteria: CriteriaWithOptionsDTO,
-) -> CriteriaWithOptions:
-    return CriteriaWithOptions(
+) -> Criteria:
+    return Criteria(
         name=criteria.name,
         description=criteria.description,
         options=[
             CriteriaOption(name=o.name, description=o.description)
             for o in criteria.options
         ],
-        prediction_field=criteria.prediction_field,
+        to_evaluate_field=criteria.to_evaluate_field,
         context_fields=criteria.context_fields,
     )
 
